package org.jtheque.books.persistence.od;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.books.persistence.od.abstraction.Author;
import org.jtheque.books.BooksModule;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.resource.IResourceManager;
import org.jtheque.core.managers.resource.ImageType;
import org.jtheque.core.managers.properties.IPropertiesManager;

import javax.swing.*;

/**
 * An author.
 *
 * @author Baptiste Wicht
 */
public final class AuthorImpl extends Author {
    private AuthorImpl memento;
    private boolean mementoState;

    @Override
    public String getAffichableText() {
        return getFirstName() + ' ' + getName();
    }

    @Override
    public String toString() {
        return getAffichableText();
    }

    @Override
    public int hashCode() {
        return Managers.getManager(IPropertiesManager.class).hashCode(this);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }

        if (obj == null) {
            return false;
        }

        if (getClass() != obj.getClass()) {
            return false;
        }

        final Author other = (Author) obj;

        if (getName() == null) {
            if (other.getName() != null) {
                return false;
            }
        } else if (!getName().equals(other.getName())) {
            return false;
        }

        if (getNote() == null) {
            if (other.getNote() != null) {
                return false;
            }
        } else if (!getNote().equals(other.getNote())) {
            return false;
        }

        if (getTheCountry() == null) {
            if (other.getTheCountry() != null) {
                return false;
            }
        } else if (!getTheCountry().equals(other.getTheCountry())) {
            return false;
        }

        if (getFirstName() == null) {
            if (other.getFirstName() != null) {
                return false;
            }
        } else if (!getFirstName().equals(other.getFirstName())) {
            return false;
        }

        return true;
    }

    @Override
    public void saveToMemento() {
        mementoState = true;

        memento = Managers.getManager(IPropertiesManager.class).createMemento(this);
    }

    @Override
    public void restoreMemento() {
        if (mementoState) {
            Managers.getManager(IPropertiesManager.class).restoreMemento(this, memento);
        }
    }

    @Override
    public Icon getIcon() {
        return Managers.getManager(IResourceManager.class).getIcon(BooksModule.IMAGE_BASE_NAME, "person", ImageType.PNG);
    }
}