package org.jtheque.books.persistence.od;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.books.BooksModule;
import org.jtheque.books.persistence.od.abstraction.Book;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.properties.IPropertiesManager;
import org.jtheque.core.managers.resource.IResourceManager;
import org.jtheque.core.managers.resource.ImageType;

import javax.swing.Icon;

/**
 * A book.
 *
 * @author Baptiste Wicht
 */
public final class BookImpl extends Book {
    private BookImpl memento;
    private boolean mementoState;

    @Override
    public String getAffichableText() {
        return getTitle();
    }

    @Override
    public String toString() {
        return getAffichableText();
    }

    @Override
    public int hashCode() {
        return Managers.getManager(IPropertiesManager.class).hashCode(this);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }

        if (obj == null) {
            return false;
        }

        if (getClass() != obj.getClass()) {
            return false;
        }

        final Book other = (Book) obj;

        if (getAuthors() == null) {
            if (other.getAuthors() != null) {
                return false;
            }
        } else if (!getAuthors().equals(other.getAuthors())) {
            return false;
        }

        if (getYear() != other.getYear()) {
            return false;
        }

        if (getPages() != other.getPages()) {
            return false;
        }

        if (getResume() == null) {
            if (other.getResume() != null) {
                return false;
            }
        } else if (!getResume().equals(other.getResume())) {
            return false;
        }

        if (getTheLending() == null) {
            if (other.getTheLending() != null) {
                return false;
            }
        } else if (!getTheLending().equals(other.getTheLending())) {
            return false;
        }

        if (getTheKind() == null) {
            if (other.getTheKind() != null) {
                return false;
            }
        } else if (!getTheKind().equals(other.getTheKind())) {
            return false;
        }

        if (getIsbn10() == null) {
            if (other.getIsbn10() != null) {
                return false;
            }
        } else if (!getIsbn10().equals(other.getIsbn10())) {
            return false;
        }

        if (getIsbn13() == null) {
            if (other.getIsbn13() != null) {
                return false;
            }
        } else if (!getIsbn13().equals(other.getIsbn13())) {
            return false;
        }

        if (getTheLanguage() == null) {
            if (other.getTheLanguage() != null) {
                return false;
            }
        } else if (!getTheLanguage().equals(other.getTheLanguage())) {
            return false;
        }

        if (getNote() == null) {
            if (other.getNote() != null) {
                return false;
            }
        } else if (!getNote().equals(other.getNote())) {
            return false;
        }

        if (getTheEditor() == null) {
            if (other.getTheEditor() != null) {
                return false;
            }
        } else if (!getTheEditor().equals(other.getTheEditor())) {
            return false;
        }

        if (getTitle() == null) {
            if (other.getTitle() != null) {
                return false;
            }
        } else if (!getTitle().equals(other.getTitle())) {
            return false;
        }

        if (getTheType() == null) {
            if (other.getTheType() != null) {
                return false;
            }
        } else if (!getTheType().equals(other.getTheType())) {
            return false;
        }

        if (getTheSaga() == null) {
            if (other.getTheSaga() != null) {
                return false;
            }
        } else if (!getTheSaga().equals(other.getTheSaga())) {
            return false;
        }

        return true;
    }

    @Override
    public void saveToMemento() {
        mementoState = true;

        memento = Managers.getManager(IPropertiesManager.class).createMemento(this);
    }

    @Override
    public void restoreMemento() {
        if (mementoState) {
            Managers.getManager(IPropertiesManager.class).restoreMemento(this, memento);
        }
    }

    @Override
    public Icon getIcon() {
        return Managers.getManager(IResourceManager.class).getIcon(BooksModule.IMAGE_BASE_NAME, "book", ImageType.PNG);
    }
}