package org.jtheque.books.services.impl;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.books.persistence.dao.able.IDaoAuthors;
import org.jtheque.books.persistence.od.AuthorImpl;
import org.jtheque.books.services.able.IAuthorsService;
import org.jtheque.books.services.able.INotesService;
import org.jtheque.core.managers.persistence.able.DataListener;
import org.jtheque.primary.services.able.ICountriesService;

import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * An authors service implementation.
 *
 * @author Baptiste Wicht
 */
public final class AuthorsService implements IAuthorsService {
    private AuthorImpl emptyAuthor;

    @Resource
    private ICountriesService countriesService;

    @Resource
    private INotesService notesService;

    @Resource
    private IDaoAuthors daoAuthors;

    @Override
    public AuthorImpl getEmptyAuthor() {
        if (emptyAuthor == null) {
            emptyAuthor = new AuthorImpl();

            emptyAuthor.setName("");
            emptyAuthor.setFirstName("");
            emptyAuthor.setNote(notesService.getDefaultNote());
            emptyAuthor.setTheCountry(countriesService.getDefaultCountry());
        }

        return emptyAuthor;
    }

    @Override
    @Transactional
    public void create(AuthorImpl author) {
        daoAuthors.create(author);
    }

    @Override
    public boolean exists(String firstname, String name) {
        return daoAuthors.exists(firstname, name);
    }

    @Override
    public AuthorImpl getAuthor(String firstname, String name) {
        return daoAuthors.getAuthor(firstname, name);
    }

    @Override
    public List<AuthorImpl> getAuthors() {
        return daoAuthors.getAuthors();
    }

    @Override
    @Transactional
    public boolean delete(AuthorImpl author) {
        return daoAuthors.delete(author);
    }

    @Override
    @Transactional
    public void save(AuthorImpl author) {
        daoAuthors.save(author);
    }

    @Override
    public List<AuthorImpl> getDatas() {
        return daoAuthors.getAuthors();
    }

    @Override
    public void addDataListener(DataListener listener) {
        daoAuthors.addDataListener(listener);
    }

    @Override
    @Transactional
    public void clearAll() {
        daoAuthors.clearAll();
    }

    @Override
    @Transactional
    public String getDataType() {
        return DATA_TYPE;
    }
}