package org.jtheque.books.view.controllers.choice;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.books.persistence.od.EditorImpl;
import org.jtheque.primary.od.LendingImpl;
import org.jtheque.primary.od.SagaImpl;
import org.jtheque.books.services.able.IEditorsService;
import org.jtheque.primary.services.able.IKindsService;
import org.jtheque.primary.services.able.ILendingsService;
import org.jtheque.primary.services.able.ISagasService;
import org.jtheque.primary.services.able.ITypesService;
import org.jtheque.books.view.controllers.undo.delete.DeletedBorrowerEdit;
import org.jtheque.books.view.controllers.undo.delete.DeletedCountryEdit;
import org.jtheque.books.view.controllers.undo.delete.DeletedEditorEdit;
import org.jtheque.primary.controller.impl.undo.DeletedKindEdit;
import org.jtheque.books.view.controllers.undo.delete.DeletedLanguageEdit;
import org.jtheque.primary.controller.impl.undo.DeletedSagaEdit;
import org.jtheque.primary.controller.impl.undo.DeletedTypeEdit;
import org.jtheque.primary.controller.impl.undo.DeletedLendingEdit;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.undo.IUndoRedoManager;
import org.jtheque.core.managers.view.IViewManager;
import org.jtheque.core.managers.language.ILanguageManager;
import org.jtheque.primary.od.*;
import org.jtheque.primary.services.able.IBorrowersService;
import org.jtheque.primary.services.able.ICountriesService;
import org.jtheque.primary.services.able.ILanguagesService;
import org.jtheque.primary.view.impl.choice.AbstractChoiceAction;

import javax.annotation.Resource;

/**
 * An action to delete the selected item.
 *
 * @author Baptiste Wicht
 */
public final class DeleteChoiceAction extends AbstractChoiceAction {

    @Resource
    private IEditorsService editorsService;

    @Resource
    private IBorrowersService borrowersService;

    @Resource
    private IKindsService kindsService;

    @Resource
    private ICountriesService countriesService;
    @Resource
    private ILanguagesService languagesService;
    @Resource
    private ILendingsService lendingsService;

    @Resource
    private ISagasService sagasService;

    @Resource
    private ITypesService typesService;

    /* Singletons */
    private final ILanguageManager resources = Managers.getManager(ILanguageManager.class);

    @Override
    public boolean canDoAction(final String action) {
        return "delete".equals(action);
    }

    @Override
    public void execute() {
        final boolean yes = Managers.getManager(IViewManager.class).askUserForConfirmation(
                resources.getMessage("choice.dialogs.delete") + ' ' + getSelectedItem().toString(),
                resources.getMessage("choice.dialogs.delete.title"));

        if (yes) {
            delete();
        }
    }

    /**
     * Delete the selected item.
     */
    private void delete() {
        if (IKindsService.DATA_TYPE.equals(getContent())) {
            boolean deleted = kindsService.delete((KindImpl) getSelectedItem());

            if (deleted) {
                Managers.getManager(IUndoRedoManager.class).addEdit(new DeletedKindEdit((KindImpl) getSelectedItem()));
            }
        } else if (IEditorsService.DATA_TYPE.equals(getContent())) {
            boolean deleted = editorsService.delete((EditorImpl) getSelectedItem());

            if (deleted) {
                Managers.getManager(IUndoRedoManager.class).addEdit(new DeletedEditorEdit((EditorImpl) getSelectedItem()));
            }
        } else if (ILendingsService.DATA_TYPE.equals(getContent())) {
            boolean deleted = lendingsService.delete((LendingImpl) getSelectedItem());

            if (deleted) {
                Managers.getManager(IUndoRedoManager.class).addEdit(new DeletedLendingEdit((LendingImpl) getSelectedItem()));
            }
        } else if (ISagasService.DATA_TYPE.equals(getContent())) {
            boolean deleted = sagasService.delete((SagaImpl) getSelectedItem());

            if (deleted) {
                Managers.getManager(IUndoRedoManager.class).addEdit(new DeletedSagaEdit((SagaImpl) getSelectedItem()));
            }
        } else if (ITypesService.DATA_TYPE.equals(getContent())) {
            boolean deleted = typesService.delete((TypeImpl) getSelectedItem());

            if (deleted) {
                Managers.getManager(IUndoRedoManager.class).addEdit(new DeletedTypeEdit((TypeImpl) getSelectedItem()));
            }
        } else if (ICountriesService.DATA_TYPE.equals(getContent())) {
            boolean deleted = countriesService.delete((CountryImpl) getSelectedItem());

            if (deleted) {
                Managers.getManager(IUndoRedoManager.class).addEdit(new DeletedCountryEdit((CountryImpl) getSelectedItem()));
            }
        } else if (ILanguagesService.DATA_TYPE.equals(getContent())) {
            boolean deleted = languagesService.delete((LanguageImpl) getSelectedItem());

            if (deleted) {
                Managers.getManager(IUndoRedoManager.class).addEdit(new DeletedLanguageEdit((LanguageImpl) getSelectedItem()));
            }
        } else if (IBorrowersService.DATA_TYPE.equals(getContent())) {
            boolean deleted = borrowersService.delete((BorrowerImpl) getSelectedItem());

            if (deleted) {
                Managers.getManager(IUndoRedoManager.class).addEdit(new DeletedBorrowerEdit((BorrowerImpl) getSelectedItem()));
            }
        }
    }
}