package org.jtheque.books.view.sort;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.books.services.able.IAuthorsService;
import org.jtheque.core.managers.persistence.Entity;
import org.jtheque.core.managers.persistence.able.DataContainerProvider;
import org.jtheque.primary.od.CountryImpl;
import org.jtheque.primary.od.abstraction.Person;
import org.jtheque.primary.services.able.ICountriesService;
import org.jtheque.primary.view.impl.models.tree.Category;
import org.jtheque.primary.view.impl.models.tree.Element;
import org.jtheque.primary.view.impl.models.tree.JThequeTreeModel;
import org.jtheque.primary.view.impl.models.tree.RootElement;
import org.jtheque.primary.view.impl.sort.Sorter;

import javax.annotation.Resource;
import java.util.List;

/**
 * This class sort by country.
 *
 * @author Baptiste Wicht
 */
public final class ByCountrySorter implements Sorter {
    @Resource
    private ICountriesService countriesService;

    @Override
    public boolean canSort(String content, String sortType) {
        return content.equals(IAuthorsService.DATA_TYPE) && sortType.equals(ICountriesService.DATA_TYPE);
    }

    @Override
    public void sort(JThequeTreeModel model) {
        RootElement root = model.getRoot();

        List<CountryImpl> groups = countriesService.getCountries();
        List<? extends Entity> contents = DataContainerProvider.getInstance().getDatas(IAuthorsService.DATA_TYPE);

        for (CountryImpl country : groups) {
            root.addCategory(new Category(country.getElementName()));
        }

        for (Entity data : contents) {
            int index = groups.indexOf(((Person) data).getTheCountry());

            root.getCategory(index).addElement((Element) data);
        }

        model.setRootElement(root);
    }
}