package org.jtheque.books.view.sort;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.books.persistence.od.EditorImpl;
import org.jtheque.books.persistence.od.abstraction.Book;
import org.jtheque.books.services.able.IBooksService;
import org.jtheque.books.services.able.IEditorsService;
import org.jtheque.core.managers.persistence.Entity;
import org.jtheque.core.managers.persistence.able.DataContainerProvider;
import org.jtheque.primary.view.impl.models.tree.Category;
import org.jtheque.primary.view.impl.models.tree.JThequeTreeModel;
import org.jtheque.primary.view.impl.models.tree.RootElement;
import org.jtheque.primary.view.impl.sort.Sorter;

import javax.annotation.Resource;
import java.util.List;

/**
 * Cette classe permet de raliser un tri par ralisateur.
 *
 * @author Baptiste Wicht
 */
public final class ByEditorSorter implements Sorter {
    @Resource
    private IEditorsService editorsService;

    @Override
    public boolean canSort(String content, String sortType) {
        return content.equals(IBooksService.DATA_TYPE) && sortType.equals(IEditorsService.DATA_TYPE);
    }

    @Override
    public void sort(JThequeTreeModel model) {
        RootElement root = model.getRoot();

        List<EditorImpl> groups = editorsService.getEditors();
        List<? extends Entity> contents = DataContainerProvider.getInstance().getDatas(IBooksService.DATA_TYPE);

        for (EditorImpl editor : groups) {
            root.addCategory(new Category(editor.getElementName()));
        }

        for (Entity data : contents) {
            Book book = (Book) data;

            if(book.getTheEditor() == null){
                root.addElement(book);
            } else {
                int index = groups.indexOf(book.getTheEditor());

                root.getCategory(index).addElement(book);
            }
        }

        model.setRootElement(root);
    }
}