package org.jtheque.books.view.sort;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.books.services.able.INotesService;
import org.jtheque.books.services.able.IAuthorsService;
import org.jtheque.core.managers.persistence.Entity;
import org.jtheque.utils.collections.ArrayUtils;
import org.jtheque.core.utils.db.DaoNotes;
import org.jtheque.core.utils.db.Note;
import org.jtheque.core.managers.persistence.able.DataContainerProvider;
import org.jtheque.primary.od.abstraction.Notable;
import org.jtheque.primary.view.impl.models.tree.Category;
import org.jtheque.primary.view.impl.models.tree.Element;
import org.jtheque.primary.view.impl.models.tree.JThequeTreeModel;
import org.jtheque.primary.view.impl.models.tree.RootElement;
import org.jtheque.primary.view.impl.sort.Sorter;

import java.util.List;

/**
 * Cette classe permet de raliser une tri par note.
 *
 * @author Baptiste Wicht
 */
public final class ByNoteSorter implements Sorter {
    private final DaoNotes daoNotes = DaoNotes.getINSTANCE();

    @Override
    public boolean canSort(String content, String sortType) {
        return content.equals(IAuthorsService.DATA_TYPE) && sortType.equals(INotesService.DATA_TYPE);
    }

    @Override
    public void sort(JThequeTreeModel model) {
        RootElement root = model.getRoot();

        Note[] groups = daoNotes.getNotes();
        List<? extends Entity> contents = DataContainerProvider.getInstance().getDatas(IAuthorsService.DATA_TYPE);

        for (Note group : groups) {
            root.addCategory(new Category(group.getElementName()));
        }

        for (Entity data : contents) {
            int index = ArrayUtils.indexOf(((Notable) data).getNote(), groups);

            root.getCategory(index).addElement((Element) data);
        }

        model.setRootElement(root);
    }
}